<?php

namespace Teknisa\FIN\Modules\Contareceber\Services;

use Teknisa\FIN\Modules\Contareceber\Factory\Rec18000CompensacaoTitulosReceberFactory;
use Doctrine\ORM\EntityManager;
use \Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\Libs\Util\NewCode;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;

class Rec18000CompensacaoTitulosReceberService
{

    const ACRESCIMO = 'A';
    const ENTRADA = 'E';
    const SAIDA = 'S';
    const EMPENHO = 'E';
    protected $crudService;
    protected $entityManager;
    protected $factory;
    protected $fluxoDeCaixaService;
    protected $newCode;
    protected $environment;
    protected $organizacao;

    public function __construct($crudService, EntityManager $entityManager, Rec18000CompensacaoTitulosReceberFactory $factory, FluxoDeCaixaService $fluxoDeCaixaService,
                                NewCode $newCode, Environment $environment)
    {
        $this->crudService = $crudService;
        $this->entityManager = $entityManager;
        $this->factory = $factory;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->organizacao = $this->environment->getNrOrgTrab();
    }

    public function geraCompensacao($rowCompensacao, $contasPagar)
    {
        if ($rowCompensacao['IDTPITUTLCOMREC'] == '0') {
            foreach ($contasPagar as $conta) {
                $this->insereAjustePagar($this->organizacao, $conta['CDFILIBPAG'], $conta['NRLANBOLPAG'], $conta['VRFINALBPAG'],
                    $rowCompensacao['CDAJUCTPAG'], $this->environment->getCdOperador(), $rowCompensacao['VRTOTALCOMREC']);
                $this->atualizaContasPagar($conta['CDEMPRESA'], $conta['CDFIMOVFLPAG'], $conta['CDCLASSFINA'], $conta['CDFILIAL'],
                    $conta['NRLANCTONF'], $conta['NRSEQVENPAG'], $conta['NRPARCELAPAG'], 'S', $conta['VRFINALBPAG'] * -1, $conta['DTMOVFLUXO'],
                    $this->organizacao, $this->environment->getCdOperador(), $rowCompensacao['DTBAIXACOMPREC']);
            }
        } else {
            foreach ($contasPagar as $conta) {
                $this->atualizaCompensacaoCredito('S', $conta['CDREDUCONT'], $conta['NRMOVICRED'], $this->organizacao);
            }
        }
        $this->insereAjusteReceber($rowCompensacao['CDFILIBOLREC'], $rowCompensacao['NRLANBOLREC'],
            $rowCompensacao['VRTOTALCOMREC'], $rowCompensacao['CDAJUCTREC'], 'I', $this->organizacao, $this->environment->getCdOperador());
        if ($rowCompensacao['VRTOTALCOMREC'] == $rowCompensacao['VRFINALBREC']) {
            $this->atualizaContasreceber($rowCompensacao['CDEMPRESA'], $rowCompensacao['CDFIMOVFLREC'], $rowCompensacao['CDCLASSFINA'],
                $rowCompensacao['CDFILIAL'], $rowCompensacao['NRLANCTONF'], $rowCompensacao['NRSEQVENREC'], $rowCompensacao['NRPARCELAREC'],
                'S', $rowCompensacao['VRORIGREC'] * -1, $rowCompensacao['DTMOVFLUXO'], $this->organizacao, $rowCompensacao['CDAJUCTREC'], $rowCompensacao['DTBAIXACOMPREC']);

        }else{
            $dtmovfluxo = self::formataData($rowCompensacao['DTMOVFLUXO']);
            $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $this->organizacao, $rowCompensacao['CDEMPRESA'], $rowCompensacao['CDFIMOVFLREC'], $rowCompensacao['CDCLASSFINA'],
                $dtmovfluxo, $rowCompensacao['VRTOTALCOMREC'] * -1, self::EMPENHO, self::SAIDA
                );
            $this->entityManager->persist($movFluxo);
            $this->entityManager->flush();
        }

        if ($rowCompensacao['IDTPITUTLCOMREC'] == '0') {
            $this->geraBaixaCompensacao($contasPagar, $rowCompensacao['IDTPITUTLCOMREC'], round($rowCompensacao['VRTOTALCOMREC'], 2),
                round($rowCompensacao['VRFINALBREC'], 2), $rowCompensacao['CDEMPRESA'], $this->environment->getCdFilial(), $rowCompensacao['CDTPBAIXAREC'],
                $rowCompensacao['DOCUMENTO'], $rowCompensacao['CDREDUCONT'], $rowCompensacao['DTBAIXACOMPREC'], $rowCompensacao['CDTPBAIXAPAG'],
                $rowCompensacao['DSBAIXAPAG'], $this->environment->getCdoperador(), $rowCompensacao['CDFILIBOLREC'], $rowCompensacao['NRLANBOLREC'],
                $this->organizacao, $this->environment->getCdOperador(), $rowCompensacao['CDCLIENTE'], $rowCompensacao['CDAJUCTPAG'], $rowCompensacao['CDAJUCTREC']);
        }else{
            $this->incluiCompensacao(
                $contasPagar, $rowCompensacao['IDTPITUTLCOMREC'], $rowCompensacao['CDEMPRESA'], $this->environment->getCdFilial(), $rowCompensacao['CDREDUCONT'],
                $rowCompensacao['CDCLIENTE'], null, $rowCompensacao['CDAJUCTREC'], null, null, $rowCompensacao['CDTPBAIXAREC'], $rowCompensacao['CDFILIBOLREC'],
                $rowCompensacao['NRLANBOLREC'], $this->environment->getCdOperador(), round($rowCompensacao['VRTOTALCOMREC'], 2), null);

        }

    }

    public function insereAjusteReceber($cdfilibrec, $nrlanbolrec, $vrtotalcomrec, $cdajuctrec, $tipoOperacao, $nrorg, $operador)
    {
        $ajusteBoletoReceber = $this->factory->getAjusteBoletoReceber($cdfilibrec, $nrlanbolrec, $cdajuctrec, $this->organizacao);
        $valorCompensado = round(floatval($vrtotalcomrec), 2);
        if ($ajusteBoletoReceber) {
            $vrajbolrec = round(floatval($ajusteBoletoReceber->getVrajbolrec()), 2);
            $valorAjustado = $valorCompensado + $vrajbolrec;
            $ajusteBoletoReceber = $this->factory->updateAjusteBoletoReceber($ajusteBoletoReceber, $valorAjustado);
            $this->entityManager->persist($ajusteBoletoReceber);
            $this->entityManager->flush();
        } else {
            $ajusteBoletoReceber = $this->factory->insereAjusteBoletoReceber($cdfilibrec, $nrlanbolrec, $cdajuctrec, $valorCompensado, 4, $nrorg, $operador);
            $this->entityManager->persist($ajusteBoletoReceber);
            $this->entityManager->flush();
        }
        $boletReceber = $this->factory->getBoletoReceber($cdfilibrec, $nrlanbolrec, $this->organizacao);
        if ($boletReceber) {
            $vrfinalbrec = $boletReceber->getVrfinalbrec() - $valorCompensado;
            $boletReceber = $this->factory->atualizaAjusteBoletoReceber($boletReceber, $vrfinalbrec, $nrorg, $operador);
            $this->entityManager->persist($boletReceber);
            $this->entityManager->flush();
        }
    }


    public function insereAjustePagar($nrorg, $cdfilibpag, $nrlanbolpag, $vrfinalbpag, $cdajuctpag, $operador, $vrtotalcomrec)
    {
        $ajusteBoletoPagar = $this->factory->getAjusteCompBoletoPag($nrorg, $cdfilibpag, $nrlanbolpag, $cdajuctpag);

        $valorCompensado = round(floatval($vrtotalcomrec), 2);

        if ($ajusteBoletoPagar) {
            $vrajbolpag = round(floatval($ajusteBoletoPagar->getVrajbolpag()), 2);
            $valorAjustado = $valorCompensado + $vrajbolpag;
            $ajusteBoletoPagar = $this->factory->updateAjusteBoletoPag($ajusteBoletoPagar, $valorAjustado);
            $this->entityManager->persist($ajusteBoletoPagar);
            $this->entityManager->flush();
        } else {
            $ajusteBoletoPagar = $this->factory->insereAjusteBoletoPag($cdfilibpag, $nrlanbolpag, $cdajuctpag, $vrfinalbpag, '4', $nrorg, $operador);
            $this->entityManager->persist($ajusteBoletoPagar);
            $this->entityManager->flush();
        }
        $boletPagar = $this->factory->getBoletosPag($cdfilibpag, $nrlanbolpag, $this->organizacao);

        if ($boletPagar) {
            $boletPagar = $this->factory->atualizaAjusteBoletoPagar($boletPagar, 0, 'S', $nrorg, $operador);
            $this->entityManager->persist($boletPagar);
            $this->entityManager->flush();
        }
    }


    public function deleteAjustePagar($nrorg, $cdfilibpag, $nrlanbolpag, $vrfinalbpag, $cdajuctpag, $operador, $vrtotalcomppag)
    {
        $ajusteBoletoPagar = $this->factory->getAjusteCompBoletoPag($nrorg, $cdfilibpag, $nrlanbolpag, $cdajuctpag);
        $valorAjustado = $ajusteBoletoPagar->getVrajbolpag();
        $valorCompensado = round(floatval($vrtotalcomppag), 2);
        if ($ajusteBoletoPagar){
            $vrajbolpag = round(floatval($ajusteBoletoPagar->getVrajbolpag()), 2);
            if ($vrajbolpag == $valorCompensado) {
                $this->entityManager->remove($ajusteBoletoPagar);
                $this->entityManager->flush();
            } else {
                $ajbolpag = $this->factory->updateAjusteBoletoPag($ajusteBoletoPagar, $valorCompensado * -1);
                $this->entityManager->persist($ajbolpag);
                $this->entityManager->flush();
            }
        }
        $boletPagar = $this->factory->getBoletosPag($cdfilibpag, $nrlanbolpag, $this->organizacao);
        if ($boletPagar) {
            $this->factory->atualizaAjusteBoletoPagar($boletPagar, $valorAjustado, 'S', $nrorg, $operador);
            $this->entityManager->persist($boletPagar);
            $this->entityManager->flush();
        }
    }

    public function atualizaContasPagar($cdempresa, $cdfimovflpag, $cdclassfina, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag,
                                        $motivo, $vrfinalbpag, $dtmovfluxo, $nrorg, $operador, $dtbaixacomp, $idstatus = 'P')
    {
        $dtmovfluxo = self::formataData($dtmovfluxo);

        $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflpag, $cdclassfina, $dtmovfluxo, $vrfinalbpag, self::EMPENHO, self::ENTRADA);
        $this->entityManager->persist($movFluxo);
        $this->entityManager->flush();
        $contaPagar = $this->factory->getContaPagar($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $nrorg);
        if ($contaPagar) {
            $contaPagar = $this->factory->updateContaPagar($contaPagar, 'E', $idstatus, $dtmovfluxo, $nrorg, $operador);
            $this->entityManager->persist($contaPagar);
            $this->entityManager->flush();
        }
        $ajusteContaPagar = $this->factory->getAjusteContaPagar($cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $nrorg);
        /*if ($ajusteContaPagar) {
            if ($ajusteContaPagar->getIdajuctpag() == 'A') {
                $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajusteContaPagar->getCdemmfajcntp(), $ajusteContaPagar->Cdfimfajcntp(),
                    $ajusteContaPagar->getCdclassfina(), $dataBaixa, $ajusteContaPagar->getVrajustepag(),
                    self::EMPENHO, self::ENTRADA);
                $this->entityManager->persist($movFluxo);
                $this->entityManager->flush();
            } else {
                $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajusteContaPagar->getCdemmfajcntp(), $ajusteContaPagar->Cdfimfajcntp(),
                    $ajusteContaPagar->getCdclassfina(), $ajusteContaPagar->getDtmovfluxo(), $ajusteContaPagar->getVrajustepag() * -1,
                    self::EMPENHO, self::ENTRADA);
                $this->entityManager->persist($movFluxo);
                $this->entityManager->flush();
            }
        }*/
    }

    public function atualizaCompensacaoCredito($idlancomcontrec, $cdreducont, $nrmovicred, $nrorg)
    {
        $movcredbanc = $this->factory->getMovimentoCredito($cdreducont, $nrmovicred, $nrorg);
        if ($movcredbanc) {
            $movcredbanc = $this->factory->atualizaCompensacaoCredito($movcredbanc, $idlancomcontrec);
            $this->entityManager->persist($movcredbanc);
            $this->entityManager->flush();
        }
    }

    public function deleteAjusteReceber($cdfilibrec, $nrlanbolrec, $vrtotalcomrec, $cdajuctrec, $tipooperacao, $nrorg, $operador)
    {
        $ajusteBoletoreceber = $this->factory->getAjusteBoletoreceber($cdfilibrec, $nrlanbolrec, $cdajuctrec, $this->organizacao);
        $valorCompensado = round(floatval($vrtotalcomrec), 2);

        if ($ajusteBoletoreceber){
            $vrajbolrec = round(floatval($ajusteBoletoreceber->getVrajbolrec()), 2);
            if ($vrajbolrec == $valorCompensado) {
                if ($ajusteBoletoreceber) {
                    $this->entityManager->remove($ajusteBoletoreceber);
                    $this->entityManager->flush();
                }
            } else {
                $ajbolrec = $this->factory->updateAjusteBoletoreceber($ajusteBoletoreceber, $valorCompensado * -1);
                $this->entityManager->persist($ajbolrec);
                $this->entityManager->flush();
            }
        }
        $boletreceber = $this->factory->getBoletoreceber($cdfilibrec, $nrlanbolrec, $this->organizacao);
        if ($boletreceber) {
            $vrfinalbrec = $boletreceber->getVrfinalbrec() - ($valorCompensado * -1);
            $boletreceber = $this->factory->atualizaAjusteBoletoreceber($boletreceber, $vrfinalbrec, $nrorg, $operador);
            $this->entityManager->persist($boletreceber);
            $this->entityManager->flush();
        }

    }

    public function atualizaContasReceber($cdempresa, $cdfimovflrec, $cdclassfina, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec,
                                        $motivo, $vrorigrec, $dtmovfluxo, $nrorg, $cdajuctrec, $dtbaixacomp, $idstatus = 'P')
    {
        $dtmovfluxo = self::formataData($dtmovfluxo);
        if ($motivo != 'S') {
            $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtmovfluxo, $vrorigrec * -1, self::EMPENHO, self::SAIDA);
            $this->entityManager->persist($movFluxo);
            $this->entityManager->flush();
        }

        $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtmovfluxo, $vrorigrec, self::EMPENHO, self::SAIDA);

        $this->entityManager->persist($movFluxo);
        $this->entityManager->flush();

        $contareceber = $this->factory->getContareceber($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $nrorg);
        if ($contareceber) {
            $contareceber = $this->factory->updateContareceber($contareceber, 'E', $idstatus, $dtmovfluxo, $this->organizacao, $this->environment->getCdoperador());
            $this->entityManager->persist($contareceber);
            $this->entityManager->flush();
        }

        $vrorigrec = round(floatval($vrorigrec), 2);
        $ajusteContareceber = $this->factory->getTipoAjusteContareceber($nrorg, $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec);
        //Atualiza fluxo da conta informada
        if ($ajusteContareceber) {
            /*$this->atualizaFluxoConta($ajusteContareceber, $cdempresa, $cdfimovflrec, $cdclassfina,
                                      $motivo, $vrorigrec, $dtmovfluxo, $nrorg, $dtbaixacomp);*/
            $ajusteContareceber = $this->factory->updateAjusteContareceber($ajusteContareceber, $dtmovfluxo, $idmovfluxo, $this->organizacao, $this->environment->getCdoperador());
            $this->entityManager->persist($ajusteContareceber);
            $this->entityManager->flush();
        }
        $contareceber = $this->factory->getAjusteContareceber($cdfilial, $nrlanctonf, date_format($dtmovfluxo, 'd/m/Y'), $cdajuctrec, $nrseqvenrec, $nrparcelarec, $this->organizacao);

        if ($contareceber) {
            $contareceber = $this->factory->updateAjusteContareceber($contareceber, 'E', $dtmovfluxo, $nrorg, $this->environment->getCdOperador());
            $this->entityManager->persist($contareceber);
            $this->entityManager->flush();
        }
    }

    /*public function atualizaFluxoConta($ajusteContareceber, $cdempresa, $cdfimovflrec, $cdclassfina,
                                        $motivo, $vrorigrec, $dtmovfluxo, $nrorg, $dtbaixacomp){
        if ($ajusteContareceber->getIdajuctrec() == self::ACRESCIMO) {
            if ($motivo != 'S') {
                // Desfaz o movimento de fluxo estimado
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtbaixacomp, $vrorigrec * -1, self::EMPENHO, self::SAIDA
                    );
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();
            }
            // Inclui o movimento de fluxo real
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtmovfluxo, $vrorigrec, self::EMPENHO, self::SAIDA);
            $this->entityManager->persist($movfluxo);
            $this->entityManager->flush();
        } else {
            if ($motivo != 'S') {
                // Desfaz o movimento de fluxo estimado
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtbaixacomp, $vrorigrec, self::EMPENHO, self::SAIDA);
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();
            }
            // Inclui o movimento de fluxo real
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $cdempresa, $cdfimovflrec, $cdclassfina, $dtmovfluxo, $vrorigrec * -1, self::EMPENHO, self::SAIDA);
            $this->entityManager->persist($movfluxo);
            $this->entityManager->flush();
        }
    }*/

    public function geraBaixaCompensacao($contasPagar, $idtpitutlcomrec, $vrtotalcomrec, $vrfinalbrec, $cdempresa,
                                         $filiproc, $cdtpbaixarec, $documento, $cdreducont, $dtbaixacomp, $cdtpbaixapag, $dsbaixapag,
                                         $cdoperlanccomcp, $cdfilibrec, $nrlanbolrec, $nrorg, $operador, $cdcliente, $cdajuctpag, $cdajuctrec)
    {
        $nrbaixarec = null;
        $nrbaixapag = null;
        if ($vrtotalcomrec == $vrfinalbrec) {
            $nrbaixarec = $this->newCode->getCodeByTableName('BAIXARECEBIDA' . $this->environment->getCdFilial(), $this->environment->getNrOrgTrab());
            $nrreciborec = $this->newCode->getCodeByTableName('NRRECIBOREC', $nrorg);
            $this->geraBaixaPlanejadarec($cdempresa, $filiproc, $cdtpbaixarec, $documento, $cdreducont, $cdoperlanccomcp, $vrtotalcomrec,
                $cdfilibrec, $nrlanbolrec, $nrorg, $operador, $nrbaixarec, $nrreciborec);
        }
        if ($idtpitutlcomrec == '0') {
            $nrbaixapag = $this->newCode->getCodeByTableName('BAIXAPAGA' . $this->environment->getCdFilial(), $this->environment->getNrOrgTrab());
            $nrrecibopag = $this->newCode->getCodeByTableName('NRRECIBOPAG', $this->environment->getNrOrgTrab());
            $this->geraBaixaPlanejadaPag($contasPagar, $cdreducont, $dtbaixacomp, $cdtpbaixapag, $dsbaixapag,
                $operador, $nrorg, $nrbaixapag, $nrrecibopag, $filiproc, $vrtotalcomrec);
        }

        $this->incluiCompensacao($contasPagar, $idtpitutlcomrec, $cdempresa, $filiproc, $cdreducont, $cdcliente, $cdtpbaixapag,
            $cdajuctrec, $nrbaixarec, $nrbaixapag, $cdtpbaixarec, $cdfilibrec, $nrlanbolrec, $cdoperlanccomcp, $vrtotalcomrec, $cdajuctpag);
    }

    public function geraBaixaPlanejadarec($cdempresa, $filiproc, $cdtpbaixarec, $documento, $cdreducont, $cdoperlanccomcp, $valorbaixa,
                                          $cdfilibrec, $nrlanbolrec, $nrorg, $operador, $nrbaixarec, $nrreciborec)
    {
        $baixaRecebida = $this->factory->insereBaixaRec(
            $cdempresa, $filiproc, $cdtpbaixarec, $documento, $cdreducont,
            $valorbaixa, $cdfilibrec, $nrlanbolrec, $nrorg, $operador, $nrbaixarec, $nrreciborec
        );
        $this->entityManager->persist($baixaRecebida);
        $boletoReceber = $this->factory->getBoletoReceber($cdfilibrec, $nrlanbolrec, $nrorg);
        if ($boletoReceber){
            $boletoReceber = $this->factory->atualizaBoletoReceber($boletoReceber, 'P', '00', $nrbaixarec, $baixaRecebida->getCdfilial(), $cdtpbaixarec, $operador, $nrorg);
            $this->entityManager->persist($boletoReceber);
            $this->entityManager->flush();
        }
    }

    public function geraBaixaPlanejadapag($contasPagar, $cdreducont, $dtbaixacomp, $cdtpbaixapag, $dsbaixapag,
                                          $operador, $nrorg, $nrbaixapag, $nrrecibopag, $filiproc, $vrtotalcomppag)
    {
        $dtbaixacomp = self::formataData($dtbaixacomp);
        $baixaPaga = $this->factory->insereBaixaPaga($contasPagar[1]['CDEMPRESA'], null, null, $nrbaixapag, $nrrecibopag, 'P', $vrtotalcomppag, $operador, $cdreducont,
            $dtbaixacomp, $filiproc, $cdtpbaixapag, $dsbaixapag, $nrorg);
        $this->entityManager->persist($baixaPaga);
        $this->entityManager->flush();
        foreach ($contasPagar as $boleto) {
            $boletosPagar = $this->factory->getBoletosPag($boleto['CDFILIBPAG'], $boleto['NRLANBOLPAG'], $nrorg);
            if ($boletosPagar) {
                $boletoPagar = $this->factory->atualizaBoletosPag($boletosPagar, 'P', $nrbaixapag, $baixaPaga->getCdfilial(), $cdtpbaixapag, $nrorg, $operador);
                $this->entityManager->persist($boletoPagar);
            }
        }
    }


    public function incluiCompensacao($contas, $idtpitutlcomrec, $cdempresa, $filiproc, $cdreducont, $cdcliente, $cdtpbaixapag,
                                      $cdajuctrec, $nrbaixarec, $nrbaixapag, $cdtpbaixarec, $cdfilibrec, $nrlanbolrec, $cdoperlanccomcp, $vrtotalcomrec, $cdajuctpag)
    {

        $nrcompensacao = $this->newCode->getCodeByTableName('COMPCONTREC'.$cdempresa, $this->environment->getNrOrgTrab());
        $compensacao = $this->factory->insereCompensacontrec($cdempresa, $nrcompensacao, $filiproc, $nrlanbolrec, $cdoperlanccomcp, $cdcliente,
            $cdtpbaixapag, $cdtpbaixarec, $cdajuctpag, $cdajuctrec, $cdreducont, $filiproc, $nrbaixapag, $cdfilibrec, $nrbaixarec,
            $idtpitutlcomrec, count($contas), $vrtotalcomrec, $this->organizacao, $this->environment->getCdOperador());
        $this->entityManager->persist($compensacao);
        $this->entityManager->flush();
        $sequencialItem = 0;
        if ($idtpitutlcomrec == '0') {
            foreach ($contas as $conta) {
                $sequencialItem = intval($sequencialItem) + 1;
                $sequencialItem = str_pad($sequencialItem, 5, '0', STR_PAD_LEFT);
                $itemCompensado = $this->factory->insereItensCompensacaoReceber($cdempresa, $nrcompensacao, $sequencialItem, $conta['CDFILIBPAG'],
                    $conta['NRLANBOLPAG'], null, null, $conta['VRFINALBPAG'], $this->organizacao, $this->environment->getCdOperador());
                $this->entityManager->persist($itemCompensado);
                $this->entityManager->flush();
            }
        } else {
            foreach ($contas as $lancamento) {
                $sequencialItem = intval($sequencialItem) + 1;
                $sequencialItem = str_pad($sequencialItem, 5, '0', STR_PAD_LEFT);
                $itemCompensado = $this->factory->insereItensCompensacaoReceber($cdempresa, $nrcompensacao, $sequencialItem, null,
                    null, $lancamento['CDREDUCONT'], $lancamento['NRMOVICRED'], $lancamento['VRMOVICRED'], $this->organizacao, $this->environment->getCdOperador());
                $this->entityManager->persist($itemCompensado);
                $this->entityManager->flush();
            }
        }
    }

    public function deleteCompensacaoCobranca($row, $contasPagar)
    {
        $boletoReceber = $this->factory->getBoletoReceber($row['CDFILIBOLREC'], $row['NRLANBOLREC'], $this->organizacao);
        if ($this->validaPermissao($row, $boletoReceber)) {
            $baixaRecebida = $this->factory->getBaixaRecebida($this->environment->getCdfilial(), $row['NRBAIXAREC'], $this->organizacao);
            $baixaPaga = $this->factory->getBaixaPaga($row['CDFILIALBAIPAG'], $row['NRBAIXAPAG'], $this->organizacao);
            $this->deleteAjusteReceber($row['CDFILIBOLREC'], $row['NRLANBOLREC'],
                $row['VRTOTALCOMREC'], $row['CDAJUCTREC'], 'I', $this->organizacao, $this->environment->getCdOperador());
            if ($row['VRTOTALCOMREC'] == $row['VRFINALBREC']) {
                $this->atualizaContasreceber($row['CDEMPRESA'], $row['CDFIMOVFLREC'], $row['CDCLASSFINA'],
                    $row['CDFILIAL'], $row['NRLANCTONF'], $row['NRSEQVENREC'], $row['NRPARCELAREC'],
                    'S', $row['VRORIGREC'], $row['DTMOVFLUXO'], $row['NRORG'], $row['CDAJUCTREC'], 'A');
            }else{
                $dtmovfluxo = self::formataData($row['DTMOVFLUXO']);
                $movFluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $this->organizacao, $row['CDEMPRESA'], $row['CDFIMOVFLREC'], $row['CDCLASSFINA'],
                    $dtmovfluxo, $row['VRTOTALCOMREC'], self::EMPENHO, self::SAIDA
                    );
                $this->entityManager->persist($movFluxo);
                    $this->entityManager->flush();
            }

            $boletoReceber = $this->factory->atualizaBoletoReceber($boletoReceber, 'A', '00', null, null, null, $this->environment->getCdOperador(), $this->organizacao);
            $this->entityManager->persist($boletoReceber);
            $this->entityManager->flush();

            if ($row['IDTPITUTLCOMREC'] == '0') {
                foreach ($contasPagar as $conta) {
                    $this->deleteAjustePagar($this->organizacao, $conta['CDFILIBPAG'], $conta['NRLANBOLPAG'], $conta['VRORIGPAG'],
                    $row['CDAJUCTPAG'], $this->environment->getCdOperador(), $row['VRTOTALCOMREC']);

                    $this->atualizaContasPagar($conta['CDEMPRESA'], $conta['CDFIMOVFLPAG'], $conta['CDCLASSFINA'], $conta['CDFILIAL'],
                    $conta['NRLANCTONF'], $conta['NRSEQVENPAG'], $conta['NRPARCELAPAG'], 'S', $conta['VRITEMCOMPREC'], $conta['DTMOVFLUXO'],
                    $this->organizacao, $this->environment->getCdOperador(), $row['DTBAIXACOMPREC']);

                    $boletoPagar = $this->factory->getBoletosPag($conta['CDFILIBPAG'], $conta['NRLANBOLPAG'], $this->organizacao);

                    if ($boletoPagar) {
                        $boletoPagar = $this->factory->atualizaBoletosPag($boletoPagar, 'A', null, null, null, $this->organizacao, $this->environment->getCdOperador());
                        $this->entityManager->persist($boletoPagar);
                        $this->entityManager->flush();
                    }
                }
            } else {
                foreach ($contasPagar as $contapag) {
                    $this->atualizaCompensacaoCredito('N', $contapag['CDREDUCONT'], $contapag['NRMOVICRED'], $this->organizacao);
                }
            }
            $itensCompensados = $this->factory->getItensCompensados($row['CDEMPRESA'], $row['NRLANCCOMPREC'], $this->organizacao);
            foreach ($itensCompensados as $item) {
                $this->entityManager->remove($item);
                $this->entityManager->flush();
            }
            $compensacao = $this->factory->getCompensacao($row['CDEMPRESA'], $row['NRLANCCOMPREC'], $this->organizacao);
            $this->entityManager->remove($compensacao);
            if ($row['IDTPITUTLCOMREC'] == '0') {
                if ($baixaRecebida) {
                    if ($baixaRecebida->getCdempresa() && $baixaRecebida->getNrlctbbrec()) {
                        $lanccont = $this->factory->getLancamentoContabil($row['CDEMPRESA'], $baixaRecebida->getNrlctbbrec(), $this->organizacao);
                        if ($lanccont) {
                            $this->entityManager->remove($lanccont);
                            $this->entityManager->flush();
                        }
                    }
                }
                if ($baixaRecebida) {
                    $this->entityManager->remove($baixaRecebida);
                    $this->entityManager->flush();
                }
                if ($baixaPaga) {
                    if ($baixaPaga->getCdempresa() && $baixaPaga->getNrlctbbpag()) {
                        $lanccont = $this->factory->getLancamentoContabil($row['CDEMPRESA'], $baixaPaga->getNrlctbbpag(), $this->organizacao);
                        if ($lanccont) {
                            $this->entityManager->remove($lanccont);
                            $this->entityManager->flush();
                        }
                    }

                    $this->entityManager->remove($baixaPaga);
                    $this->entityManager->flush();
                }
            }
        }
    }


    public function validaPermissao($row, $boletReceber)
    {
        if ((($boletReceber->getIdstatbrec() == 'B') || ($boletReceber->getIdstatbrec() == 'P'))
            && $row['NRBAIXAREC'] == ''
        ) {
            throw new \Exception('Exclusão não permitida! Boleto a receber ligado à compensação já foi baixado.');
        }
        if (($boletReceber->getIdsitucnabr() <> '00') && ($boletReceber->getIdsitucnabr() <> '01')) {
            throw new \Exception('Exclusão não permitida! Boleto a receber ligado à compensação foi enviado via CNAB.');
        }
        return true;
    }

    public static function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}
?>